/*
Software to interface an FT232RL to a SGP18T display
(C) 2012 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <ftdi.h>
#include <gd.h>

struct ftdi_context ftdi;
unsigned char sendbuff[10240*1024];
int sendpos;

//Masks for the pin signals
#define LCD_A0  0x10 //DTR, pin 2
#define LCD_SDI 0x02 //RX, pin 5
#define LCD_SCK 0x01 //TX, pin 1
#define LCD_CS  0x08 //CTS, pin 11

//H & W of lcd. We use it in portrait mode here.
#define LCD_H 48+1
#define LCD_W 288

//Error handling routine.
int error(const char *err) {
	printf("Error at %s: %s\n", err, ftdi_get_error_string(&ftdi));
	exit(0); //Dirty. Disable to continue after errors.
	return 0;
}

//Send a SPI byte
int spiSend(unsigned char byte, int isData) {
	unsigned char b=0;
	int x;
	if (isData) b|=LCD_A0;
	for (x=0; x<8; x++) {
		if (byte&0x80) b|=LCD_SDI; else b&=~LCD_SDI;
		sendbuff[sendpos++]=b;
		sendbuff[sendpos++]=b|LCD_SCK;
		byte<<=1;
	}
}

//Raise CS
void spiDone(void) {
	sendbuff[sendpos++]=LCD_CS;
}

//Actually output all queued commands
void spiFlush() {
//	sendbuff[sendpos++]=0;
	if (ftdi_write_data(&ftdi, sendbuff, sendpos)<0) error("writing buffer");
	sendpos=0;
}

//Init the LCD
void lcdInit() {
	int x;
//	spiSend(0xae, 0);
//	spiDone();
	spiSend(0xa8, 0);
	spiSend(48, 0);
	spiDone();
	spiSend(0xa0, 0);
	spiSend(0x70, 0);
	spiDone();
	spiSend(0xaf, 0);
//	spiSend(0x86, 0);
//	spiDone();

	for (x=0x81; x<0x84; x++) {
	spiSend(x, 0);
	spiSend(0x80, 0);
	spiDone();
	}

	spiSend(0x15, 0);
	spiSend(0, 0);
	spiSend(96, 0);
	spiDone();
	spiSend(0x75,0);
	spiSend(0, 0);
	spiSend(48, 0);
	spiDone();


	for (x=0; x<10240; x++) {
		spiSend(0xFF,1);
	}
	spiDone();
	spiFlush();
}

//Show the png-file called filename on the LCD
void lcdShowImage(char* filename) {
	FILE *f;
	int x, y, p1, p2, p3;
	unsigned int col;
	gdImagePtr img;
	f=fopen(filename,"r");
	if (f==NULL) {
		perror(filename);
		return;
	}
	img=gdImageCreateFromPng(f);
	fclose(f);
	if (img==NULL) {
		printf("Couldn't load image: %s\n", filename);
		return;
	}
	spiSend(0x15, 0);
	spiSend(0, 0);
	spiSend(96-1, 0);
	spiDone();
	spiSend(0x75,0);
	spiSend(0, 0);
	spiSend(48, 0);
	spiDone();


	for (y=0; y<LCD_H; y++) {
		for (x=LCD_W-1; x>=0; x-=3) {
			p1=gdImageGreen(img,gdImageGetPixel(img, x, y))>>3;
			p2=gdImageGreen(img,gdImageGetPixel(img, x+1, y))>>3;
			p3=gdImageGreen(img,gdImageGetPixel(img, x+2, y))>>3;
			col=(p1<<11)|(p2<<6)|(p3);
			spiSend(col>>8, 1);
			spiSend(col&0xff, 1);
		}
	}
	spiDone();
	spiFlush();
}

//Main function
int main(int argc, char **argv) {
	int x;
	//Open FTDI communications
	if (ftdi_init(&ftdi)<0) return error("init");
	if (ftdi_usb_open(&ftdi, 0x0403, 0x6001)<0) return error("open");
	if (ftdi_set_bitmode(&ftdi, LCD_A0|LCD_SDI|LCD_SCK|LCD_CS, BITMODE_BITBANG)<0) error("bitmode");
	if (ftdi_set_baudrate(&ftdi, 3000000/4)<0) error("baudrate");
	ftdi_usb_purge_buffers(&ftdi);
	sendpos=0;
	//Init lcd
	lcdInit();
	//Send all pngs on the command line
	for (x=1; x<argc; x++) {
		lcdShowImage(argv[x]);
	}
}
